# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# gestion des autorisations sur les onglets
#
###########################################################################

"""
permission manager
gestions des autorisations sur les onglets (thumbs)
Deux modes existent :
     - le mode fichiers : des .ini pour les profiles et permissions
     - le mode base de données : utilise db_mapper
    Si les fichiers existent c'est le mode fichier

Un InitPermission et InitPermissionDB charge l'ensemble des associations
profiles<->règle ou les règles sont liées à des onglets et item de bureau

Un UserContext charge les liens entre attributs utilisateurs et profiles
Il fournit les méthodes de connexion d'utilisateur:
    - user = usercontext.connect_user(nom, attributs)
    - onglet = usercontext.get_thumbs(user)
"""

from os.path import isfile
from posh.perm.profile import User
from posh.perm.profile import Profile
from posh.perm.rule import Rule, Thumb, DesktopItem
from posh.perm.error import *
from sets import Set

from posh.config.config import profile_filename, perm_filename

if isfile(profile_filename) and isfile(perm_filename):
    from posh.perm.cfg_parser import PermissionParser, ProfilerParser
else:
    from posh.backend import db_mapper

def debug(msg):
    f_ = file("/tmp/usercontext.txt", 'a')
    f_.write(msg)
    f_.write('\n')
    f_.close()

def test_value(value):
    """ teste si la variable est une valeur (pas un dico) """
    return type(value) != dict

def format_value(value):
    """ formatte une valeur en liste """
    if type(value) != list:
        if hasattr(value, '__iter__'):
            return list(value)
        else:
            return [value]
    return value

def linearize(dico, linear={}):
    """
        linéarise un dico
        {'etab':{'rne':'123456'}, 'name':['toto']}
        devient
        {'rne':['123456'}, 'name':['toto']}
    """
    B = linear
    for key, value in dico.items():
        if test_value(value):
            B[key] = format_value(value)
        else:
            B.update(linearize(value))
    return B

class UserContext:
    """
        recupere les données utilisateur et les profils
        qui leur sont associés soi depuis un .ini (si il est renseigné)
        sinon depuis une bdd
    """
    def __init__(self, profile_filename=None):
        print "Initialisation du contexte utilisateur"
        self.filename = profile_filename
        self.load_profiles()

    def load_profiles(self):
        print "++++ (re) Loading profiles....."
        if self.filename: # on est en mode fichier
            self.parse_profile(self.filename)
        else: # on est en mode DB
            self.get_db_profiles()
        self.profiles = PERMS.get_profiles()
        print "++++ Fin du reload"

    def parse_profile(self, filename=profile_filename):
        """
            parse un fichier .ini de profils
        """
        parser = ProfilerParser()
        parser.parse_file(filename)
        # valeurs del liens issues des .ini
        self.type_user = parser.get_values()

    def reload(self):
        """
            reload le contexte utilisateur
        """
        self.load_profiles()

    def get_db_profiles(self):
        """
            récupère les profils dans une base de données
        """
        self.type_user = db_mapper.get_all_profiles()

    def get_user_profiles(self, user_datas):
        """
            récupère les profils du user
            en fonction de ses attributs envoyés depuis le sso
        """
        # applatissage et formatage du dico :
        user_datas = linearize(user_datas)

        profiles = ['main_user']
        for key, value in user_datas.items():
            if self.type_user.has_key(key):
                for val in value:
                    if self.type_user[key].has_key(val):
                        typed_profiles = self.type_user[key][val]
                        if hasattr(typed_profiles, '__iter__'): #cas où plusieurs roles pour la même clé
                            profiles.extend(typed_profiles)
                        else:
                            profiles.append(typed_profiles)
                    elif self.type_user[key].has_key('None'): # cas d"'une valeur None pour l'attribut utilisateur
                        typed_profiles = self.type_user[key]["None"]
                        if hasattr(typed_profiles, '__iter__'): #cas où plusieurs roles pour la même clé
                            profiles.extend(typed_profiles)
                        else:
                            profiles.append(typed_profiles)
        return profiles

    def populate_user(self, user):
        """
            'peuple' l'utilisateur en attribuant
            les objets profiles à l'utilisateur
        """
        profiles_libelle = self.get_user_profiles(user.user_datas)
        self._populate_user(user, profiles_libelle)

    def connect_user(self, username, user_datas):
        """
            renvoie un avatar pour le user username
        """
        user = User(username, user_datas)
        self.populate_user(user)
        return user

    def is_allowed(self, user, thumb_name):
        """
            permet de savoir si un utilisateur a le droit sur un onglet
            thumb: une instance d'onglet
            user : un utilisateur
        """
        for profile in user.profiles:
            for rule in self.profiles[profile.name].get_rules():
                for thumb in rule.thumbs:
                    if thumb.name == thumb_name:
                        return True
        return False

    def is_allowed_desktop(self, user, desktop_item_name):
        """
            permet de savoir si un utilisateur a le droit sur un bouton du bureau
            desktop_item: une instance d'item de bureau
            user : un utilisateur
        """
        for profile in user.profiles:
            for rule in profile.get_rules():
                for desktop_item in rule.desktop_items:
                    if desktop_item.name == desktop_item_name:
                        return True
        return False

    def get_thumbs(self, user):
        """
        liste des onglets disponibles pour un utilisateur
        """
        authorized_thumbs = Set()
        for profile in user.profiles:
            for rule in profile.get_rules():
                authorized_thumbs = authorized_thumbs.union(rule.thumbs)
        return list(authorized_thumbs)

    def get_desktop_items(self, user):
        """ liste les boutons du bureau autorisés pour l'utilisateur
        """
        authorized_desktop_items = Set()
        for profile in user.profiles:
            for rule in profile.get_rules():
                authorized_desktop_items = authorized_desktop_items.union(rule.desktop_items)
        return list(authorized_desktop_items)

    def _populate_user(self, user, profiles):
        """
            user: objet user
            profiles: liste de nom de profil
        """
        for profile in profiles:
            if self.profiles.has_key(profile):
                user.add_profile(self.profiles[profile])
            else:
                # logger l'inextistence du profil
                pass
        return user

class InitDBPermissions:
    """ construit les objets permissions
        depuis une entrée base de données
    """

    def get_profiles(self):
        """
            - une entrée profile : dict(id= , name=, description=)
                id ne sert que lors du contact à la base
                (on le perd ensuite lors du passage en objet)

        """
        # recuperation des profiles et creation des objets
        profiles = db_mapper.get_profile()
        self.profiles = dict([(profile['name'], Profile(profile['name'], description=profile['description'])) for profile in profiles if profile])

        # recuperation des onglets des profils
        for profile in profiles:
            thumbs = self.get_thumbs(profile['profileid'])
            desktop_items = self.get_desktop_items(profile['profileid'])
            rule = Rule()
            rule.add_thumbs(thumbs)
            rule.add_desktop_items(desktop_items)
            self.profiles[profile['name']].add_rule(rule)
        return self.profiles


    def get_thumbs(self, profile_id):
        """ renvoie les objets thumbs du profil profil
        """
        thumbs = [Thumb(thumb['name'],
                  description=thumb['description'],
                  seq=thumb['indice'],
                  _type=thumb['poshtype'],
                  param=thumb['url'],
                  admin_profile_id=thumb.get('admin_profile_id', '-1'))
                  for thumb in db_mapper.get_profilethumb(profile_id) if thumb]
        return thumbs

    def get_desktop_items(self, profile_id):
        """ récupère les items de bureau"""
        desktop_items = [DesktopItem(desktop_item['name'],
                         libelle=desktop_item['libelle'],
                         url=desktop_item['url'],
                         icon=desktop_item['icon'])
                         for desktop_item in db_mapper.get_profiledesktopitem(profile_id) if desktop_item]
        return desktop_items

class InitPermissions:
    """
        construit les objets permissions
        depuis un fichier .ini
        WARNING : En désuétude, le passage en bdd a entrainé des modifications
                  non reportés (some troubleshooting could occur)
    """
    def __init__(self, filename):
        """
        filename: les permissions sont pour l'instant stockées dans un fichier
        """
        if isfile(filename):
            self.filename = filename
        else:
            raise FileNotFoundError, "file '%s' does not exist" % filename
        self.parse_file()

    def parse_file(self):
        """parse un fichier .ini
        """
        parser = PermissionParser()
        parser.parse_file(self.filename)
        self.parser = parser

    def get_profiles(self):
        """construction des profils,
        un profil ayant un attribut rules,
        il faut récupérer les règles de permissions avant
        """
        # profils
        self.profiles = dict([(name, Profile(name, description=descr)) for name, descr
                                    in self.parser.get_profiles().items()])
        # mise à jour des rules dans les onglets
        #         ** s'assurer que get_thumbs a été lancé (self.thumbs existe)
        self.get_thumbs()
        for profile, thumbs in self.parser.get_permissions().items():
            # ajoutons des règles plutôt que des onglets directement
            # objet Rule
            rule = Rule()
            thumbs = [self.thumbs[thumb] for thumb in thumbs if thumb in self.thumbs.keys()]
            rule.add_thumbs(thumbs)
            self.profiles[profile].add_rule(rule)

        # gestion des boutons d'application pour le bureau utilisateur
        self.get_desktop_items()
        for profile, desktop_items in self.parser.get_permissions().items():
            rule = Rule()
            desktop_items = [self.desktop_items[item] for item in desktop_items if item in self.desktop_items.keys()]
            rule.add_desktop_items(desktop_items)
            self.profiles[profile].add_rule(rule)

        # mise à jour des profils de base
        for profile, baseprofiles in self.parser.get_baseprofiles().items():
            for baseprofile in baseprofiles:
                self.profiles[profile].add_baseprofile(self.profiles[baseprofile])
        return self.profiles

    def get_thumbs(self):
        # onglets
        self.thumbs = dict([(name, Thumb(name,
                                         description=datas['description'],
                                         seq=datas['seq'],
                                         _type=datas['_type'],
                                         param=datas['param'])) for name, datas
                                    in self.parser.get_thumbs().items()])
        return self.thumbs

    def get_desktop_items(self):
        """ récupère les items de bureau"""
        self.desktop_items = dict([(name, DesktopItem(name,
                                                      libelle=datas['libelle'],
                                                      url=datas['url'],
                                                      icon=datas['icon'])) for name, datas
                                                      in self.parser.get_desktop_items().items()])
        return self.desktop_items

def perm_manager(filename):
    """point d'entrée, fonction de création du gestionnaire de permissions   """
    if isfile(filename):
        perms = InitPermissions(filename)
        return perms
    else:
        return InitDBPermissions()

def user_context(filename=profile_filename):
    """
        point d'entrée, fonction de création du gestionnaire de contexte utilisateur
    """
    if isfile(filename):
        context = UserContext(filename)
        return context
    else:
        return UserContext()

PERMS = perm_manager(perm_filename)
USERCONTEXT = user_context()
