# -*- coding: UTF-8 -*-
"""
    Outil d'authentification
    PamAuth permet l'auth PAM
    AuthenticatedPage est une page Nevow garantissant l'authentification:
    class Page(AuthenticatedPage):
        ...
        def locatechild(self, ctx, segments):
            session = inevow.ISession(ctx)
            request = inevow.IRequest(ctx)
            # authentification
            root_url = _get_root_url(ctx)
            togo = self.authenticated(request, session, root_url)
            if togo:
               return self.auth_redirect(togo)
        ...
"""
import PAM
from time import time
import xmlrpclib
from os.path import join

from twisted.web.resource import Resource
from twisted.python import log
from twisted.internet.task import LoopingCall

from nevow.rend import Page
from nevow.inevow import ISession, IRequest
try:
    from posh.config.config import SESSION_TIMEOUT
except:
    #timeout de la session de l'interface en 12 min si pas d'action
    SESSION_TIMEOUT = 12 * 60

#fréquence de vérification de la session
LIFETIME = 60

# TODO: voir avec Bruno pour les id de sessions au lieu de faire une session avec timer (ce serait mieux quand même), pamcas ?

class PamAuth():
    """AUTHENTIFICATION PAM
    """
    def __init__(self, service='passwd'):
        self.service = service
        self.auth = PAM.pam()

    def pam_conv(self, auth, query_list, userData):
        """fonction de dialogue avec la lib_pam"""
        resp = []
        for i in range(len(query_list)):
            query, type = query_list[i]
            if type == PAM.PAM_PROMPT_ECHO_ON:
                val = raw_input(query)
                resp.append((val, 0))
            elif type == PAM.PAM_PROMPT_ECHO_OFF:
                resp.append((self.passwd, 0))
            elif type == PAM.PAM_PROMPT_ERROR_MSG or type == PAM.PAM_PROMPT_TEXT_INFO:
                print query
                resp.append(('', 0))
            else:
                return None
        return resp

    def authenticate(self, user, passwd):
        self.auth.start(self.service)
        self.passwd = passwd
        if user != None:
            self.auth.set_item(PAM.PAM_USER, user)
        self.auth.set_item(PAM.PAM_CONV, self.pam_conv)
        try:
            self.auth.authenticate()
            self.auth.acct_mgmt()
            self.passwd = None
        except PAM.error, resp:
            return False
        except:
            print 'Internal error'
            return False
        else:
            return True


class SessionTimer:
    """
       timer de session nevow
       session : objet ISession nevow
       timeout : timeout de la session
       lifetime : fréquence de contrôle
    """
    def __init__(self, session, timeout=SESSION_TIMEOUT, lifetime=LIFETIME):
        self.session = session
        self.sessionTimeout = timeout
        self.touch()
        self.looping = LoopingCall(self.check_expiration)
        self.looping.start(lifetime, now=False)

    def touch(self):
        """ remet à jour le timer
            une action a été effectuée
        """
        self.lastModified = self._getTime()

    def _getTime(self):
        """ renvoie le temps courant
        """
        return time()

    def check_expiration(self):
        """
            teste l'expiration de la session
        """
        log.msg("Check session expiration")
        if self._getTime() - self.lastModified > self.sessionTimeout:
            self.looping.stop()
            self.session.kill_myself(self.session)

class Redirection(Resource):
    """
        Objet de redirection:
            redirection html
            redirection ajax : se contente de renvoyer la chaine 'refresh'
                                (géré par le js client (handleResponse de 1_ajax.js)
            @_type : type de redirection (ajax ou autre chose pour html)
            @url : url vers laquelle redirigé
    """
    def __init__(self, _type, url):
        Resource.__init__(self)
        self.url = url
        self._type = _type

    def redirect_html(self, request):
        log.msg("Redirection to %s" % self.url)
        request.redirect(self.url)
        return """<html>
                <head><meta http-equiv=\"refresh\" content=\"0;URL=%(url)s\"></head>
                <body bgcolor=\"#FFFFFF\" text=\"#000000\"><a href=\"%(url)s\">click here</a></body>
                </html>
                """ % {'url': self.url}

    def redirect_ajax(self):
        """ redirection pour les requêtes ajax """
        log.msg("Ajax redirection to rootpage")
        return """refresh"""

    def render(self, request):
        if self._type == "ajax":
            return self.redirect_ajax()
        else:
            return self.redirect_html(request)

    def getChild(self, name, request):
        return self


class AuthenticatedPage(Page):
    """
        Page authentifiée
        auth_form : adresse du formulaire d'authentification
        auth_server : adresse du serveur xmlrpc d'authentification

        class MyPage(AuthenticatedPage):
            ...
            def locatechild(self, ctx, segments):
                session = inevow.ISession(ctx)
                request = inevow.IRequest(ctx)
                # authentification
                root_url = _get_root_url(ctx)
                togo = self.authenticated(request, session, root_url)
                if togo:
                   return self.auth_redirect(togo)
            ...

        gère un timer de session, les valeurs sont initialisées en dur dans le code
    """

    def __init__(self, auth_form, auth_server):
        self.auth_server = auth_server
        self.auth_form = auth_form
        super(AuthenticatedPage, self).__init__()

    def _init_ssoparams(self, session):
        """ renvoie le client xmlrpc pour la connexion au sso
        """
        def kill_myself(sess):
            print "Session Timed out: killing the session"
            delattr(session, 'timer')
            sess.expire()

        if not hasattr(session, "sso"):
            session.sso = xmlrpclib.ServerProxy(join(self.auth_server, 'xmlrpc'))
        if not hasattr(session, "is_logged_in"):
            session.is_logged_in = None

        # timer de session
        if not hasattr(session, "kill_myself"):
            session.kill_myself = kill_myself
        if not hasattr(session, "timer"):
            log.msg("Init session Timer")
            session.timer = SessionTimer(session)
        else:
            log.msg("Refreshing the session timer")
            session.timer.touch()

    def reset_session(self, session):
        if hasattr(session, 'is_logged_in'):
            delattr(session, 'is_logged_in')
        if hasattr(session, 'timer'):
            session.timer.looping.stop()
        if hasattr(session, 'kill_myself'):
            session.kill_myself(session)
            delattr(session, "kill_myself")

    def auth_redirect(self, redirection):
        """ renvoie la redirection vers redirection
        """
        return redirection

    def _get_request_type(self, request):
        """ renvoie le type de requête à laquelle il faut répondre
            si la requete à un x-requested-with de valeur XMLHttpRequest dans son header:
                c'est une requête ajax,
            sinon il s'agit d'une requête html classique
        """
        if request.getHeader('x-requested-with') == "XMLHttpRequest":
            print "It's an ajax request"
            return "ajax"
        else:
            print "It's an html request"
            return 'html'

    def authenticated(self, request, session, root_url):
        """
            authentifie la session de l'utilisateur, 3 situations:
            1 - la session est authentifiée
            2 - on a un ticket , on récupère les infos utilisateurs,
                on valide les droits et on marque la session comme authentifiée
            3 - on redirige vers le sso pour authentification
        """
        self._init_ssoparams(session)
        redirect_type = self._get_request_type(request)
        # 1 -
        if session.is_logged_in:
            log.msg("+++ Session authentifiée pour %s" % session.is_logged_in)
            return None
        # 2 -
        if request.args.has_key('ticket'):
            log.msg("+++ On a un ticket SSO ")
            app_ticket = request.args['ticket'][0]
            datas = session.sso.get_user_details(app_ticket, root_url)
            if datas[0]:
                uid = datas[1]['uid'][0]
                if uid == 'admin':
                    session.is_logged_in = uid
                    log.msg("===> Le ticket d'application a été validé, l'utilisateur %s est autorisé à accéder à l'application." % uid)
                    return Redirection(redirect_type,
                                        root_url), ()
                else:
                    return Redirection(redirect_type,
                                        join(self.auth_form, 'logout')), ()
        # 3 -
        log.msg("+++ On est pas authentifié")
        return Redirection(redirect_type,
                            join(self.auth_form, '?service=%s' % root_url)), ()
